@@ -1,13 +1,13 @@
+package Bio::Das::Lite;
 #########
-# Author:        rpettett@cpan.org
-# Maintainer:    rpettett@cpan.org
+# Author:        rmp@sanger.ac.uk
+# Maintainer:    rmp@sanger.ac.uk
 # Created:       2005-08-23
-# Last Modified: $Date: 2009/01/12 10:29:26 $ $Author: rmp $
-# Source:        $Source: /cvsroot/Bio-DasLite/Bio-DasLite/lib/Bio/Das/Lite.pm,v $
-# Id:            $Id: Lite.pm,v 1.61 2009/01/12 10:29:26 rmp Exp $
+# Last Modified: $Date: 2007/02/20 14:31:54 $ $Author: rmp $
+# Source:        $Source $
+# Id:            $Id $
 # $HeadURL $
 #
-package Bio::Das::Lite;
 use strict;
 use warnings;
 use Bio::Das::Lite::UserAgent;
@@ -16,50 +16,29 @@ use HTTP::Headers;
 use SOAP::Lite;
 use Carp;
 use English qw(-no_match_vars);
-use Readonly;
 
 our $DEBUG    = 0;
-our $VERSION  = do { my @r = (q$Revision: 1.61 $ =~ /\d+/mxg); sprintf '%d.'.'%03d' x $#r, @r };
-Readonly::Scalar our $BLK_SIZE => 8192;
-Readonly::Scalar our $TIMEOUT  => 5;
-Readonly::Scalar our $MAX_REQ  => 5;
-Readonly::Scalar our $MAX_HOST => 7;
-Readonly::Scalar our $LINKRE   => qr{<link\s+href="([^"]+)"[^>]*?>([^<]*)</link>}mix;
-Readonly::Scalar our $NOTERE   => qr{<note[^>]*>([^<]*)</note>}mix;
-
-BEGIN {
-  if(!LWP::Protocol->can('parse_head')) {
-    require LWP::Protocol;
-    *LWP::Protocol::parse_head = sub { shift->_elem('parse_head', @_); };
-  }
-}
+our $VERSION  = do { my @r = (q$Revision: 1.47 $ =~ /\d+/mxg); sprintf '%d.'.'%03d' x $#r, @r };
+our $BLK_SIZE = 8192;
+our $TIMEOUT  = 5;
+our $MAX_REQ  = 5;
 
 #########
 # $ATTR contains information about document structure - tags, attributes and subparts
 # This is split up by call to reduce the number of tag passes for each response
 #
 our %COMMON_STYLE_ATTRS = (
-			   yoffset        => [], # WTSI extension (available in Ensembl)
-			   scorecolormin  => [], # WTSI extension
-			   scorecolormax  => [], # WTSI extension
-			   scoreheightmin => [], # WTSI extension
-			   scoreheightmax => [], # WTSI extension
-			   zindex         => [], # WTSI extension (available in Ensembl)
-			   width          => [], # WTSI extension (available in Ensembl)
-			   height         => [],
-			   fgcolor        => [],
-			   bgcolor        => [],
-			   label          => [],
-			   bump           => [],
-			  );
-our %SCORED_STYLE_ATTRS = (
-			   min            => [],
-			   max            => [],
-			   steps          => [],
-			   color1         => [],
-			   color2         => [],
-			   color3         => [],
-			   height         => [],
+			   'yoffset'        => [], # WTSI extension (available in Ensembl)
+			   'scorecolormin'  => [], # WTSI extension
+			   'scorecolormax'  => [], # WTSI extension
+			   'scoreheightmin' => [], # WTSI extension
+			   'scoreheightmax' => [], # WTSI extension
+			   'zindex'         => [], # WTSI extension (available in Ensembl)
+			   'height'         => [],
+			   'fgcolor'        => [],
+			   'bgcolor'        => [],
+			   'label'          => [],
+			   'bump'           => [],
 			  );
 our $ATTR     = {
 		 '_segment'     => {
@@ -84,12 +63,6 @@ our $ATTR     = {
 		 'sequence'     => {
 				    'sequence'     => [qw(id start stop moltype version)],
 				   },
-		 'dna'          => {
-				    'sequence'     => {
-						       'sequence' => [qw(id start stop version)],
-						       'dna'      => [qw(length)],
-						      },
-				   },
 		 'entry_points' => {
 				    'entry_points' => [qw(href version)],
 				    'segment'      => {
@@ -115,12 +88,11 @@ our $ATTR     = {
                                                                                   dbSource
                                                                                   dbVersion
                                                                                   dbAccessionId
-                                                                                  dbCoordSys)],
+                                                                                  dbCoodSys)],
 						       'alignobjectdetail' => {
 									       'alignobjectdetail' => [qw(dbSource
                                                                                                           property)],
 									      },
-						       'sequence'          => [],
 						      },
 				    'score'        => [qw(score)],
 				    'block'        => {
@@ -168,7 +140,7 @@ our $ATTR     = {
                                                                                      atomName
                                                                                      x y z)]
 								      },
-							  },
+							  }, 
 					      },
 				 'connect' => {
 					       'connect' => [qw(atomSerial type)],
@@ -180,16 +152,15 @@ our $ATTR     = {
 		 'sources' => {
 			       'source' => {
 					    'source'     => [qw(uri title doc_href description)],
-					    'maintainer' => {
+					    'maintainer' => { 
 							     'maintainer' => [qw(email)],
 							    },
 					    'version'    => {
-							     'version'     => [qw(uri created)],
-							     'coordinates' => {
+							     'version'     => [qw(uri created)], 
+							     'coordinates' => { 
 									       'coordinates' => [qw(uri
                                                                                                     source
                                                                                                     authority
-                                                                                                    taxid
                                                                                                     test_range
                                                                                                     version)],
 									      },
@@ -209,83 +180,70 @@ our $ATTR     = {
 						     'type'     => {
 								    'type'  => [qw(id)],
 								    'glyph' => {
-                                                                                'glyph'          => [qw(zoom)],
 										'arrow'          => {
+												     %COMMON_STYLE_ATTRS,
 												     'parallel'     => [],
 												     'bar_style'    => [], # WTSI extension
-												     %COMMON_STYLE_ATTRS,
 												    },
 										'anchored_arrow' => {
+												     %COMMON_STYLE_ATTRS,
 												     'parallel'     => [],
 												     'orientation'  => [], # WTSI extension
 												     'no_anchor'    => [], # WTSI extension
 												     'bar_style'    => [], # WTSI extension
-												     %COMMON_STYLE_ATTRS,
 												    },
 										'box'            => {
+												     %COMMON_STYLE_ATTRS,
 												     'linewidth'    => [],
 												     'pattern'      => [],  # WTSI extension
-												     %COMMON_STYLE_ATTRS,
 												    },
 										'farrow'         => {                      # WTSI extension
+												     %COMMON_STYLE_ATTRS,
 												     'orientation'  => [],
 												     'no_anchor'    => [],
 												     'bar_style'    => [], # WTSI extension
-												     %COMMON_STYLE_ATTRS,
 												    },
 										'rarrow'         => {                      # WTSI extension
+												     %COMMON_STYLE_ATTRS,
 												     'orientation'  => [],
 												     'no_anchor'    => [],
 												     'bar_style'    => [], # WTSI extension
-												     %COMMON_STYLE_ATTRS,
 												    },
 										'cross'          => {
-												     'linewidth'    => [],  # WTSI extension
 												     %COMMON_STYLE_ATTRS,
+												     'linewidth'    => [],  # WTSI extension
 												    },
 										'dot'            => \%COMMON_STYLE_ATTRS,
 										'ex'             => {
-												     'linewidth'    => [],  # WTSI extension
 												     %COMMON_STYLE_ATTRS,
+												     'linewidth'    => [],  # WTSI extension
 												    },
 										'hidden'         => \%COMMON_STYLE_ATTRS,
 										'line'           => {
-												     'style'        => [],
 												     %COMMON_STYLE_ATTRS,
+												     'style'        => [],
 												    },
 										'span'           => {
-												     'bar_style'    => [], # WTSI extension
 												     %COMMON_STYLE_ATTRS,
+												     'bar_style'    => [], # WTSI extension
 												    },
 										'text'           => {
+												     %COMMON_STYLE_ATTRS,
 												     'font'         => [],
 												     'fontsize'     => [],
 												     'string'       => [],
 												     'style'        => [],
-												     %COMMON_STYLE_ATTRS,
 												    },
 										'primers'        => \%COMMON_STYLE_ATTRS,
 										'toomany'        => {
-												     'linewidth'    => [],
 												     %COMMON_STYLE_ATTRS,
+												     'linewidth'    => [],
 												    },
 										'triangle'       => {
+												     %COMMON_STYLE_ATTRS,
 												     'linewidth'    => [],
 												     'direction'    => [],
 												     'orientation'  => [],
-												     %COMMON_STYLE_ATTRS,
-												    },
-										'gradient'       => {
-												     %SCORED_STYLE_ATTRS,
-												    },
-										'histogram'      => {
-												     %SCORED_STYLE_ATTRS,
-												    },
-										'tiling'         => {
-												     %SCORED_STYLE_ATTRS,
-												    },
-										'lineplot'       => {
-												     %SCORED_STYLE_ATTRS,
 												    },
 									       },
 								   },
@@ -297,10 +255,9 @@ our $ATTR     = {
 # $OPTS contains information about parameters to use for queries
 #
 our $OPTS = {
-	     'feature'      => [qw(segment type category categorize feature_id group_id maxbins)],
+	     'feature'      => [qw(segment type category categorize feature_id group_id)],
 	     'type'         => [qw(segment type)],
 	     'sequence'     => [qw(segment)],
-	     'dna'          => [qw(segment)],
 	     'entry_points' => [],
 	     'dsn'          => [],
 	     'stylesheet'   => [],
@@ -324,7 +281,7 @@ sub new {
 
   if($ref && ref $ref) {
     for my $arg (qw(dsn timeout caching callback registry user_agent
-                    http_proxy proxy_user proxy_pass no_proxy)) {
+                    http_proxy proxy_user proxy_pass)) {
       if(exists $ref->{$arg} && $self->can($arg)) {
 	$self->$arg($ref->{$arg});
       }
@@ -344,7 +301,6 @@ sub new_from_registry {
   return $self;
 }
 
-# We implement this method because LWP does not parse user/password
 sub http_proxy {
   my ($self, $proxy) = @_;
   if($proxy) {
@@ -356,7 +312,7 @@ sub http_proxy {
     $self->{'_checked_http_proxy_env'} = 1;
   }
 
-  if($self->{'http_proxy'} =~ m{^(https?://)(\S+):(.*?)\@(.*?)$}mx) {
+  if($self->{'http_proxy'} =~ m|^(https?://)(\S+):(.*?)\@(.*?)$|mx) {
     #########
     # http_proxy contains username & password - we'll set them up here:
     #
@@ -369,29 +325,11 @@ sub http_proxy {
   return $self->{'http_proxy'};
 }
 
-sub no_proxy {
-  my ($self, @args) = @_;
-
-  if (scalar @args) {
-    if ($args[0] && ref $args[0] && ref $args[0] eq 'ARRAY') {
-      $self->{'no_proxy'} = $args[0];
-    } else {
-      $self->{'no_proxy'} = \@args;
-    }
-  }
-
-  if(!$self->{'_checked_no_proxy_env'}) {
-    $self->{'no_proxy'} ||= [split /\s*,\s*/mx, $ENV{'no_proxy'} || q()];
-    $self->{'_checked_no_proxy_env'} = 1;
-  }
-
-  return $self->{'no_proxy'} || [];
-}
-
 sub _get_set {
   my ($self, $key, $value) = @_;
   if(defined $value) {
     $self->{$key} = $value;
+#print STDERR qq(set self $key = @{[$value||'undef']}\n);
   }
   return $self->{$key};
 }
@@ -421,11 +359,6 @@ sub caching {
   return $self->_get_set('caching', $val);
 }
 
-sub max_hosts {
-  my ($self, $val) = @_;
-  return $self->_get_set('_max_hosts', $val);
-}
-
 sub max_req {
   my ($self, $val) = @_;
   return $self->_get_set('_max_req', $val);
@@ -443,7 +376,7 @@ sub basename {
   my @res          = ();
 
   for my $service (@dsns) {
-    $service =~ m{(https?://.*/das)/?}mx;
+    $service =~ m|(https?://.*/das)/?|mx;
     if($1) {
       push @res, $1;
     }
@@ -499,11 +432,6 @@ sub sequence {
   return $self->_generic_request($query, 'sequence', $opts);
 }
 
-sub dna {
-  my ($self, $query, $opts) = @_;
-  return $self->_generic_request($query, 'dna', $opts);
-}
-
 sub alignment {
   my ($self, $opts) = @_;
   return $self->_generic_request($opts, 'alignment');
@@ -542,44 +470,15 @@ sub _generic_request {
   my ($self, $query, $fname, $opts) = @_;
   $opts       ||= {};
   delete $self->{'currentsegs'};
+  my $ref       = {};
+  my $dsn       = $opts->{'use_basename'}?$self->basename():$self->dsn();
+  my @bn        = @{$dsn};
   my $results   = {};
+  my @queries   = ();
   my $reqname   = $fname;
   $reqname      =~ s/[\(\)]//mxg;
   ($fname)      = $fname =~ /^([a-z_]+)/mx;
-
-  my $ref       = $self->build_requests({
-					 query   => $query,
-					 fname   => $fname,
-					 reqname => $reqname,
-					 opts    => $opts,
-					 results => $results
-					});
-
-  $self->_fetch($ref, $opts->{'headers'});
-  $DEBUG and print {*STDERR} qq(Content retrieved\n);
-
-  $self->postprocess($fname, $results);
-
-  #########
-  # deal with caching
-  #
-  if($self->{'caching'}) {
-    $DEBUG and print {*STDERR} qq(Performing cache handling\n);
-    for my $s (keys %{$results}) {
-      if($DEBUG && !$results->{$s}) {
-	print {*STDERR} qq(CACHE HIT for $s\n); ## no critic (InputOutput::RequireCheckedSyscalls)
-      }
-      $results->{$s}          ||= $self->{'_cache'}->{$s};
-      $self->{'_cache'}->{$s} ||= $results->{$s};
-    }
-  }
-
-  return $results;
-}
-
-sub build_queries {
-  my ($self, $query, $fname) = @_;
-  my @queries;
+  my $attr      = $ATTR->{$fname};
 
   if($query) {
     if(ref $query eq 'HASH') {
@@ -605,7 +504,7 @@ sub build_queries {
 	#########
 	# ... or if the first array arg is a hash, stitch the series of queries together
 	#
-	push @queries, map { ## no critic
+	push @queries, map {
 	  my $q = $_;
 	  join q(;), map { "$_=$q->{$_}" } grep { $q->{$_} } @{$OPTS->{$fname}};
 	} @{$query};
@@ -630,40 +529,13 @@ sub build_queries {
     #
     push @queries, q();
   }
-  return \@queries;
-}
-
-sub _hack_fname {
-  my ($self, $fname) = @_;
-  #########
-  # Sucky hacks
-  #
-  if($fname eq 'structure') {
-    $fname = 'dasstructure';
-  } elsif($fname eq 'dna') {
-    $fname = 'sequence';
-  }
-  return $fname;
-}
-
-sub build_requests {
-  my ($self, $args) = @_;
-  my $query     = $args->{query};
-  my $fname     = $args->{fname};
-  my $reqname   = $args->{reqname};
-  my $opts      = $args->{opts};
-  my $results   = $args->{results};
-  my $queries   = $self->build_queries($query, $fname);
-  my $attr      = $ATTR->{$fname};
-  my $dsn       = $opts->{'use_basename'}?$self->basename():$self->dsn();
-  my @bn        = @{$dsn};
-  my $ref       = {};
 
   for my $bn (@bn) {
     #########
     # loop over dsn basenames
     #
-    for my $request (map { "$bn/$reqname?$_" } @{$queries}) {
+
+    for my $request (map { "$bn/$reqname?$_" } @queries) {
       #########
       # and for each dsn, loop over the query request
       #
@@ -677,6 +549,7 @@ sub build_requests {
       }
 
       $results->{$request} = [];
+
       $ref->{$request}     = sub {
 	my $data                     = shift;
 	$self->{'data'}->{$request} .= $data;
@@ -686,59 +559,52 @@ sub build_requests {
 	  # If we haven't yet found segment information for this request
 	  # Then look for some. This one is a non-destructive scan.
 	  #
-	  my $matches = $self->{'data'}->{$request}  =~ m{(<segment[^>]*>)}mix;
+	  my $matches = $self->{'data'}->{$request}  =~ m!(<segment[^>]*>)!mix;
 
 	  if($matches) {
 	    my $seginfo = [];
-	    $self->_parse_branch({
-				  request    => $request,
-				  seginfo    => $seginfo,
-				  attr       => $ATTR->{'_segment'},
-				  blk        => $1,
-				  addseginfo => 0,
-				 });
+	    _parse_branch($self,
+			  $request,
+			  $seginfo,
+			  $ATTR->{'_segment'},
+			  $1,
+			  0);
 	    $self->{'currentsegs'}->{$request} = $seginfo->[0];
 	  }
 	}
 
 	if($DEBUG) {
-	  print {*STDERR} qq(invoking _parse_branch for $fname\n); ## no critic (InputOutput::RequireCheckedSyscalls)
+	  print {*STDERR} qq(invoking _parse_branch for $fname\n);
 	}
 
-	#########
-	# Sucky hacks
-	#
-	if($fname eq 'dna') {
-	  $attr  = $attr->{'sequence'};
+	if($fname eq 'structure') {
+	  #########
+	  # Sucky hack because the structure response uses 'dasstructure' tags
+	  # Whoever did that needs a kicking
+	  #
+	  $fname = 'dasstructure';
 	}
-	$fname = $self->_hack_fname($fname);
 
-	my $pat = qr{(<$fname.*?/$fname>|<$fname[^>]+/>)}smix;
+	my $pat = qr!(<$fname.*?/$fname>|<$fname[^>]+/>)!smix;
 	while($self->{'data'}->{$request} =~ s/$pat//mx) {
-	  $self->_parse_branch({
-				request    => $request,
-				seginfo    => $results->{$request},
-				attr       => $attr,
-				blk        => $1,
-				addseginfo => 1,
-			       });
+	  _parse_branch($self, $request, $results->{$request}, $attr, $1, 1);
 	}
 
 	if($DEBUG) {
-	  print {*STDERR} qq(completed _parse_branch\n); ## no critic (InputOutput::RequireCheckedSyscalls)
+	  print {*STDERR} qq(completed _parse_branch\n);
 	}
 
 	return;
       };
     }
   }
-  return $ref;
-}
 
-sub postprocess {
-  my ($self, $fname, $results) = @_;
+  $self->_fetch($ref, $opts->{'headers'});
+  $DEBUG and print {*STDERR} qq(Content retrieved\n);
 
-  $fname = $self->_hack_fname($fname);
+  #########
+  # Postprocessing hacks
+  #
 
   #########
   # Add in useful segment information for empty segments
@@ -753,53 +619,46 @@ sub postprocess {
   }
 
   #########
-  # fix ups
+  # minor tidy up for entry_points requests
   #
   if($fname eq 'entry_points') {
     $DEBUG and print {*STDERR} qq(Running postprocessing for entry_points\n);
-
     for my $s (keys %{$results}) {
       my $res = $results->{$s} || [];
       for my $r (@{$res}) {
 	delete $r->{'segment_id'};
       }
     }
+  }
 
-  } elsif($fname eq 'sequence') {
-    $DEBUG and print {*STDERR} qq(Running postprocessing for dna\n);
-
+  #########
+  # deal with caching
+  #
+  if($self->{'caching'}) {
+    $DEBUG and print {*STDERR} qq(Performing cache handling\n);
     for my $s (keys %{$results}) {
-      my $res = $results->{$s} || [];
-
-      for my $r (@{$res}) {
-	if(exists $r->{'dna'}) {
-	  $r->{'dna'} =~ s/\s+//smgx;
-
-	} elsif(exists $r->{'sequence'}) {
-	  $r->{'sequence'} =~ s/\s+//smgx;
-	}
+      if($DEBUG && !$results->{$s}) {
+	print {*STDERR} qq(CACHE HIT for $s\n);
       }
+      $results->{$s}          ||= $self->{'_cache'}->{$s};
+      $self->{'_cache'}->{$s} ||= $results->{$s};
     }
   }
-  return;
+
+  return $results;
 }
 
 #########
 # Set up the parallel HTTP fetching
-# This uses our LWP::Parallel::UserAgent subclass which handles DAS statuses
+# This uses our LWP::Parallel::UserAgent subclass which has better proxy handling
 #
 sub _fetch {
   my ($self, $url_ref, $headers) = @_;
-
-  if (!$self->{'ua'}) {
-    $self->{'ua'} = Bio::Das::Lite::UserAgent->new();
-    $self->{'ua'}->proxy( ['http','https'], $self->http_proxy() );
-    $self->{'ua'}->no_proxy( @{ $self->no_proxy() } );
-  }
-
+  $self->{'ua'}                ||= Bio::Das::Lite::UserAgent->new(
+								'http_proxy' => $self->http_proxy(),
+							       );
   $self->{'ua'}->initialize();
-  $self->{'ua'}->max_req  ($self->max_req()   || $MAX_REQ );
-  $self->{'ua'}->max_hosts($self->max_hosts() || $MAX_HOST);
+  $self->{'ua'}->max_req($self->max_req()||$MAX_REQ);
   $self->{'statuscodes'}          = {};
   $headers                      ||= {};
   $headers->{'X-Forwarded-For'} ||= $ENV{'HTTP_X_FORWARDED_FOR'};
@@ -828,11 +687,12 @@ sub _fetch {
   $DEBUG and print {*STDERR} qq(Requests submitted. Waiting for content\n);
   eval {
     $self->{'ua'}->wait($self->{'timeout'});
-
-  } or do {
-    carp $EVAL_ERROR;
   };
 
+  if($EVAL_ERROR) {
+    carp $EVAL_ERROR;
+  }
+
   for my $url (keys %{$url_ref}) {
     if(ref $url_ref->{$url} ne 'CODE') {
       next;
@@ -864,14 +724,9 @@ sub statuscodes {
 # recursively parse the XML blocks and build the corresponding response data structure
 #
 sub _parse_branch {
-  my ($self, $args) = @_;
-  my $dsn           = $args->{request};
-  my $ar_ref        = $args->{seginfo};
-  my $attr          = $args->{attr};
-  my $blk           = $args->{blk};
-  my $addseginfo    = $args->{addseginfo};
-  my $depth         = $args->{depth} || 0;
-  my $ref           = {};
+  my ($self, $dsn, $ar_ref, $attr, $blk, $addseginfo, $depth) = @_;
+  $depth ||= 0;
+  my $ref  = {};
 
   my (@parts, @subparts);
   while(my ($k, $v) = each %{$attr}) {
@@ -888,16 +743,9 @@ sub _parse_branch {
   for my $subpart (@subparts) {
     my $subpart_ref  = [];
 
-    my $pat = qr{(<$subpart[^>]*/>|<$subpart[^>]*?(?!/)>.*?/$subpart>)}smix;
+    my $pat = qr!(<$subpart[^>]*/>|<$subpart[^>]*?(?\!/)>.*?/$subpart>)!smix;
     while($blk =~ s/$pat//mx) {
-      $self->_parse_branch({
-			    request    => $dsn,
-			    seginfo    => $subpart_ref,
-			    attr       => $attr->{$subpart},
-			    blk        => $1,
-			    addseginfo => 0,
-			    depth      => $depth+1,
-			   });
+      _parse_branch($self, $dsn, $subpart_ref, $attr->{$subpart}, $1, 0, $depth+1);
     }
 
     if(scalar @{$subpart_ref}) {
@@ -918,13 +766,13 @@ sub _parse_branch {
     my $opts = $attr->{$tag}||[];
 
     for my $a (@{$opts}) {
-      ($tmp)              = $blk =~ m{<$tag[^>]+$a="([^"]+?)"}smix;
+      ($tmp)              = $blk =~ m|<$tag[^>]+$a="([^"]+?)"|smix;
       if(defined $tmp) {
 	$ref->{"${tag}_$a"} = $tmp;
       }
     }
 
-    ($tmp) = $blk =~ m{<$tag[^>]*>([^<]+)</$tag>}smix;
+    ($tmp) = $blk =~ m|<$tag[^>]*>([^<]+)</$tag>|smix;
     if(defined $tmp) {
       $tmp         =~ s/^\s+$//smgx;
       if(length $tmp) {
@@ -932,11 +780,34 @@ sub _parse_branch {
       }
     }
     if($tmp && $DEBUG) {
-      print {*STDERR} q( )x($depth*2), qq(  $tag = $tmp\n); ## no critic (InputOutput::RequireCheckedSyscalls)
+      print {*STDERR} q( )x($depth*2), qq(  $tag = $tmp\n);
     }
   }
 
-  $self->_parse_twig($dsn, $blk, $ref, $addseginfo);
+  #########
+  # handle multiples of twig elements here
+  #
+  my $linkre = qr!<link\s+href="([^"]+)"[^>]*?>([^<]*)</link>!mix;
+  my $notere = qr!<note[^>]*>([^<]*)</note>!mix;
+  $blk       =~ s!$linkre!{
+                           $ref->{'link'} ||= [];
+                           push @{$ref->{'link'}}, {
+                                                    'href' => $1,
+                                                    'txt'  => $2,
+                                                   };
+                           q()
+                          }!smegix;
+  $blk       =~ s!$notere!{
+                           $ref->{'note'} ||= [];
+                           push @{$ref->{'note'}}, $1;
+                           q()
+                          }!smegix;
+
+  if($addseginfo && $self->{'currentsegs'}->{$dsn}) {
+    while(my ($k, $v) = each %{$self->{'currentsegs'}->{$dsn}}) {
+      $ref->{$k} = $v;
+    }
+  }
 
   push @{$ar_ref}, $ref;
   $DEBUG and print {*STDERR} q( )x($depth*2), qq(leaving _parse_branch\n);
@@ -954,34 +825,6 @@ sub _parse_branch {
   return q();
 }
 
-sub _parse_twig {
-  my ($self, $dsn, $blk, $ref, $addseginfo) = @_;
-
-  #########
-  # handle multiples of twig elements here
-  #
-  $blk =~ s/$LINKRE/{
-                     $ref->{'link'} ||= [];
-                     push @{$ref->{'link'}}, {
-                                              'href' => $1,
-                                              'txt'  => $2,
-                                             };
-                     q()
-                    }/smegix;
-  $blk =~ s/$NOTERE/{
-                     $ref->{'note'} ||= [];
-                     push @{$ref->{'note'}}, $1;
-                     q()
-                    }/smegix;
-
-  if($addseginfo && $self->{'currentsegs'}->{$dsn}) {
-    while(my ($k, $v) = each %{$self->{'currentsegs'}->{$dsn}}) {
-      $ref->{$k} = $v;
-    }
-  }
-  return;
-}
-
 sub registry {
   my ($self, @reg) = @_;
 
@@ -999,13 +842,11 @@ sub registry_sources {
   my ($self, $filters, $flush) = @_;
 
   $filters       ||= {};
-  my $category     = $filters->{'category'}   || [];
-  my $capability   = $filters->{'capability'} || [];
-
+  my $category     = $filters->{'category'}   ||[];
+  my $capability   = $filters->{'capability'} ||[];
   if(!ref $category) {
     $category = [$category];
   }
-
   if(!ref $capability) {
     $capability = [$capability];
   }
@@ -1017,23 +858,16 @@ sub registry_sources {
       $DEBUG and print {*STDERR} qq(Building soap request for $reg\n);
       my $soap = SOAP::Lite->service("$reg:das_directory?wsdl");
       $DEBUG and print {*STDERR} qq(Setting soap proxy\n);
-
       if($self->http_proxy()) {
 	$soap->proxy($reg, proxy => ['http'=>$self->http_proxy()]);
       }
-
       $DEBUG and print {*STDERR} qq(Running request for $reg\n);
 
       $SIG{ALRM} = sub { croak 'timeout'; };
-      alarm $self->timeout();
-
+      alarm $TIMEOUT;
       eval {
 	push @{$self->{'_registry_sources'}}, @{$soap->listServices()};
-
-      } or do {
-	carp $EVAL_ERROR;
       };
-
       alarm 0;
     }
   }
@@ -1045,7 +879,7 @@ sub registry_sources {
     return $self->{'_registry_sources'};
   }
 
-  my $sources = $self->{'_registry_sources'};
+  my @sources = @{$self->{'_registry_sources'}};
 
   #########
   # Apply capability filter
@@ -1053,8 +887,17 @@ sub registry_sources {
   if((ref $capability eq 'ARRAY') &&
      (scalar @{$capability})) {
     my $str    = join q(|), @{$capability};
-    my $match  = qr/$str/mx;
-    $sources = [grep { $self->_filter_capability($_, $match) } @{$sources}];
+    my $match  = qr/$str/;
+    my $filter = sub {
+      my ($src, $match) = @_;
+      for my $scap (@{$src->{'capabilities'}}) {
+	if($scap =~ $match) {
+	  return 1;
+	}
+      }
+      return 0;
+    };
+    @sources = grep { &{$filter}($_, $match) } @sources;
   }
 
   #########
@@ -1062,30 +905,19 @@ sub registry_sources {
   #
   if((ref $category eq 'ARRAY') &&
      (scalar @{$category})) {
-    $sources  = [grep { $self->_filter_category($_, $category) } @{$sources}];
+    my $filter = sub {
+      my ($src, $match) = @_;
+      for my $scoord (@{$src->{'coordinateSystem'}}) {
+	for my $m (@{$match}) {
+	  return 1 if($scoord->{'category'} eq $m);
+	}
+      }
+      return 0;
+    };
+    @sources  = grep { &{$filter}($_, $category) } @sources;
   }
 
-  return $sources;
-}
-
-sub _filter_capability {
-  my ($self, $src, $match) = @_;
-  for my $scap (@{$src->{'capabilities'}}) {
-    if($scap =~ $match) {
-      return 1;
-    }
-  }
-  return 0;
-};
-
-sub _filter_category {
-  my ($self, $src, $match) = @_;
-  for my $scoord (@{$src->{'coordinateSystem'}}) {
-    for my $m (@{$match}) {
-       return 1 if($scoord->{'category'} eq $m);
-    }
-  }
-  return 0;
+  return \@sources;
 }
 
 1;
@@ -1121,7 +953,6 @@ Bio::Das::Lite - Perl extension for the DAS (HTTP+XML) Protocol (http://biodas.o
  Options can be: dsn        (optional scalar or array ref, URLs of DAS services)
                  timeout    (optional int,      HTTP fetch timeout in seconds)
                  http_proxy (optional scalar,   web cache or proxy if not set in %ENV)
-                 no_proxy   (optional list/ref, non-proxiable domains if not set in %ENV)
                  caching    (optional bool,     primitive caching on/off)
                  callback   (optional code ref, callback for processed XML blocks)
                  registry   (optional array ref containing DAS registry service URLs
@@ -1166,14 +997,6 @@ For a complete list of capabilities and categories, see:
 
     $das->proxy_pass('secretpassword');
 
-=head2 no_proxy : Get/Set domains to not use proxy for
-
-    $das->no_proxy('ebi.ac.uk', 'localhost');
-    OR
-    $das->no_proxy( ['ebi.ac.uk', 'localhost'] );
-    
-    Always returns an arrayref
-
 =head2 user_agent : Get/Set user-agent for request headers
 
     $das->user_agent('GroovyDAS/1.0');
@@ -1264,9 +1087,8 @@ For a complete list of capabilities and categories, see:
                                       # optional args - see DAS Spec
                                      });
   my $feature_data4 = $das->features([
-                                      {'segment'  => '1:1,1000000','type' => 'karyotype',},
-                                      {'segment'  => '2:1,1000000',},
-                                      {'group_id' => 'OTTHUMG00000036084',},
+                                      {'segment' => '1:1,1000000','type' => 'karyotype',},
+                                      {'segment' => '2:1,1000000',},
                                      ]);
 
   #########
@@ -1286,9 +1108,6 @@ For a complete list of capabilities and categories, see:
   # or:
   $das->features(['1:1,1000000', '2:1,1000000', '3:1,1000000'], $callback);
 
-  # or:
-  $das->features([{'group_id' => 'OTTHUMG00000036084'}, '2:1,1000000', '3:1,1000000'], $callback);
-
 =head2 alignment : Retrieve protein alignment data for a query.  This can be a multiple sequence alignment
                     or pairwise alignment.  Note - this has not been tested for structural alignments as there
                     is currently no Das source avialable.
@@ -1317,12 +1136,7 @@ For a complete list of capabilities and categories, see:
   my $code         = $das->statuscodes($url);
   my $code_hashref = $das->statuscodes();
 
-=head2 max_hosts set number of running concurrent host connections
-
-  $das->max_hosts(7);
-  print $das->max_hosts();
-
-=head2 max_req set number of running concurrent requests per host
+=head2 max_req set number of running concurrent requests
 
   $das->max_req(5);
   print $das->max_req();
@@ -1345,18 +1159,6 @@ For a complete list of capabilities and categories, see:
     'category' => ['Protein Sequence'],
   });
 
-=head2 build_queries
-
-Constructs an arrayref of DAS requests including parameters for each call
-
-=head2 build_requests
-
-Constructs the LWP::P::UA callbacks
-
-=head2 postprocess
-
-Applies processing to the result set, e.g. removal of whitespace from sequence responses.
-
 =head1 DESCRIPTION
 
 This module is an implementation of a client for the DAS protocol (XML over HTTP primarily for biological-data).
@@ -1379,8 +1181,6 @@ This module is an implementation of a client for the DAS protocol (XML over HTTP
 
 =head1 INCOMPATIBILITIES
 
-Versions of LWP from 5.816 onwards (at time of writing upto 5.820) break LWP::Parallel and
-therefore Bio::Das::Lite. It is recommended to use version 5.814 of LWP.
 
 =head1 BUGS AND LIMITATIONS
 
@@ -1399,11 +1199,11 @@ The DAS Registry at:
 
 =head1 AUTHOR
 
-Roger Pettett, E<lt>rpettett@cpan.orgE<gt>
+Roger Pettett, E<lt>rmp@sanger.ac.ukE<gt>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (C) 2007 GRL, by Roger Pettett
+Copyright (C) 2005 GRL, by Roger Pettett
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.4 or,
